'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var documenter = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(options) {
    var _this = this;

    var tiers, pack, headers, documentationUrl, tarball, metadata, tgz, creds, auth, s3, s3Stream, upload, uploadPromise;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            headers = function headers(name, dir) {
              return { name: path.join(dir || '', name) };
            };

            options = _.defaults({}, options, {
              referenceUrl: 'https://docs.taskcluster.net/reference/',
              aws: null,
              credentials: undefined,
              project: null,
              tier: null,
              schemas: {},
              menuIndex: 10,
              readme: path.join(rootdir.get(), 'README.md'),
              docsFolder: path.join(rootdir.get(), '/docs'),
              bucket: 'taskcluster-raw-docs',
              references: [],
              publish: process.env.NODE_ENV == 'production'
            });

            assert(options.schemas, 'options.schemas must be given');
            assert(options.tier, 'options.tier must be given');
            tiers = ['core', 'platform', 'integrations', 'operations', 'libraries', 'workers'];

            assert(tiers.indexOf(options.tier) !== -1, 'options.tier must be one of ' + tiers.join(', '));

            if (!options.project) {
              pack = require(path.join(rootdir.get(), 'package.json'));

              options.project = pack.name;
            }

            documentationUrl = options.referenceUrl + options.tier + '/' + options.project;
            tarball = tar.pack();
            metadata = {
              version: 1,
              project: options.project,
              tier: options.tier,
              menuIndex: options.menuIndex
            };


            tarball.entry(headers('metadata.json'), (0, _stringify2.default)(metadata, null, 2));

            _.forEach(options.schemas, function (schema, name) {
              return tarball.entry(headers(name, 'schemas'), schema);
            });

            _.forEach(options.references, function (reference) {
              return tarball.entry(headers(reference.name + '.json', 'references'), (0, _stringify2.default)(reference.reference, null, 2));
            });

            _context2.prev = 13;
            _context2.t0 = tarball;
            _context2.t1 = headers('README.md');
            _context2.next = 18;
            return fs.readFile(options.readme);

          case 18:
            _context2.t2 = _context2.sent;

            _context2.t0.entry.call(_context2.t0, _context2.t1, _context2.t2);

            _context2.next = 27;
            break;

          case 22:
            _context2.prev = 22;
            _context2.t3 = _context2['catch'](13);

            if (!(_context2.t3.code !== 'ENOENT')) {
              _context2.next = 26;
              break;
            }

            throw _context2.t3;

          case 26:
            debug('README.md does not exist. Continuing.');

          case 27:
            _context2.prev = 27;
            _context2.next = 30;
            return _promise2.default.all(recursiveReadSync(options.docsFolder).map(function () {
              var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(file) {
                var relativePath;
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        relativePath = path.relative(options.docsFolder, file);
                        _context.t0 = tarball;
                        _context.t1 = headers(relativePath, 'docs');
                        _context.next = 5;
                        return fs.readFile(file, { encoding: 'utf8' });

                      case 5:
                        _context.t2 = _context.sent;

                        _context.t0.entry.call(_context.t0, _context.t1, _context.t2);

                      case 7:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, _this);
              }));

              return function (_x2) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 30:
            _context2.next = 37;
            break;

          case 32:
            _context2.prev = 32;
            _context2.t4 = _context2['catch'](27);

            if (!(_context2.t4.code !== 'ENOENT')) {
              _context2.next = 36;
              break;
            }

            throw _context2.t4;

          case 36:
            debug('Docs folder does not exist. Continuing.');

          case 37:

            tarball.finalize();

            tgz = tarball.pipe(zlib.createGzip());

            if (!options.publish) {
              _context2.next = 54;
              break;
            }

            creds = options.aws;

            if (creds) {
              _context2.next = 46;
              break;
            }

            auth = new client.Auth({
              credentials: options.credentials,
              baseUrl: options.authBaseUrl
            });
            _context2.next = 45;
            return auth.awsS3Credentials('read-write', options.bucket, options.project + '/');

          case 45:
            creds = _context2.sent;

          case 46:
            s3 = new aws.S3(creds.credentials);
            s3Stream = S3UploadStream(s3);
            upload = s3Stream.upload({
              Bucket: options.bucket,
              Key: options.project + '/latest.tar.gz'
            });

            // handle progress

            upload.on('part', function (details) {
              debug(details);
            });

            uploadPromise = new _promise2.default(function (resolve, reject) {
              // handle upload completion
              upload.on('uploaded', function (details) {
                debug(details);
                resolve(details);
              });

              // handle errors
              upload.on('error', function (error) {
                console.log(error.stack);
                reject(error);
              });
            });

            // pipe the incoming filestream through compression and up to s3

            tgz.pipe(upload);
            _context2.next = 54;
            return uploadPromise;

          case 54:
            return _context2.abrupt('return', {
              tgz: tgz,
              documentationUrl: documentationUrl
            });

          case 55:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this, [[13, 22], [27, 32]]);
  }));

  return function documenter(_x) {
    return _ref.apply(this, arguments);
  };
}();

var downloader = function () {
  var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(options) {
    var auth, creds, s3, readStream;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            options = _.defaults({}, options, {
              credentials: {},
              bucket: 'taskcluster-raw-docs',
              project: null
            });

            auth = new client.Auth({
              credentials: options.credentials
            });
            _context3.next = 4;
            return auth.awsS3Credentials('read-only', options.bucket, options.project + '/');

          case 4:
            creds = _context3.sent;
            s3 = new aws.S3(creds.credentials);
            readStream = s3.getObject({
              Bucket: options.bucket,
              Key: options.project + '/latest.tar.gz'
            }).createReadStream();
            return _context3.abrupt('return', readStream.pipe(zlib.Unzip()));

          case 8:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function downloader(_x3) {
    return _ref3.apply(this, arguments);
  };
}();

// Export documenter as top-level function, it's the only thing used by most
// people importing this library.


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var tar = require('tar-stream');
var _ = require('lodash');
var assert = require('assert');
var fs = require('mz/fs');
var path = require('path');
var recursiveReadSync = require('recursive-readdir-sync');
var zlib = require('zlib');
var rootdir = require('app-root-dir');
var aws = require('aws-sdk');
var client = require('taskcluster-client');
var S3UploadStream = require('s3-upload-stream');
var debug = require('debug')('taskcluster-lib-docs');

documenter.downloader = downloader;
documenter.documenter = documenter; // For backwards compatibility
module.exports = documenter;
//# sourceMappingURL=docs.js.map
